/*
 * cluto.h 
 *
 * This file contains function prototypes and constant definitions 
 * for CLUTO
 *
 * Started 11/23/01
 * George
 *
 */

#ifndef __cluto_h__
#define __cluto_h__


#ifndef _MSC_VER
#define __cdecl
#endif

/*------------------------------------------------------------------------
* Function prototypes 
*-------------------------------------------------------------------------*/

#ifdef __cpluplus
extern "C" {
#endif

void __cdecl CLUTO_VP_ClusterDirect(int nrows, int ncols, int *rowptr, int *rowind,
              float *rowval, int simfun, int crfun, int rowmodel, int colmodel, 
              float colprune, int ntrials, int niter, int seed, int dbglvl, 
              int nparts, int *part);


void __cdecl CLUTO_VP_ClusterRB(int nrows, int ncols, int *rowptr, int *rowind,
              float *rowval, int simfun, int crfun, int rowmodel, int colmodel, 
              float colprune, int ntrials, int niter, int seed, 
              int kwayrefine, int dbglvl, int nparts, int *part);


void __cdecl CLUTO_VA_Cluster(int nrows, int ncols, int *rowptr, int *rowind,
              float *rowval, int simfun, int crfun, int rowmodel, int colmodel, 
              float colprune, int dbglvl, int nparts, int *part, int *ptree, 
              float *tsims);


void __cdecl CLUTO_SA_Cluster(int nrows, int *rwgts, float *smat, int crfun, 
              int memflag, int dbglvl, int nparts, int *part, int *ptree, 
              float *tsims, float *gain);


float __cdecl CLUTO_V_GetSolutionQuality(int nrows, int ncols, int *rowptr, int *rowind,  
               float *rowval, int simfun, int crfun, int rowmodel, int colmodel, 
               float colprune, int nparts, int *part);


void __cdecl CLUTO_V_GetClusterStats(int nrows, int ncols, int *rowptr, int *rowind,
              float *rowval, int simfun, int rowmodel, int colmodel, float colprune, 
              int nparts, int *part, int *pwgts, 
	      float *cintsim, float *cintsdev, float *izscores, 
	      float *cextsim, float *cextsdev, float *ezscores);


void __cdecl CLUTO_V_GetClusterFeatures(int nrows, int ncols, int *rowptr,
              int *rowind, float *rowval, int simfun, int rowmodel, int colmodel, 
              float colprune, int nparts, int *part, int nfeatures,
              int *internalids, float *internalwgts, int *externalids,
              float *externalwgts);


void __cdecl CLUTO_V_BuildTree(int nrows, int ncols, int *rowptr, int *rowind,
              float *rowval, int simfun, int crfun, int rowmodel, int colmodel, 
              float colprune, int treetype, int dbglvl, int nparts, int *part, 
              int *ptree, float *tsims);


void __cdecl CLUTO_V_GetTreeStats(int nrows, int ncols, int *rowptr, int *rowind,
              float *rowval, int simfun, int rowmodel, int colmodel, float colprune, 
              int nparts, int *part, int *ptree, int *pwgts, float *cintsim, 
              float *cextsim);


void __cdecl CLUTO_V_GetTreeFeatures(int nrows, int ncols, int *rowptr, int *rowind,
              float *rowval, int simfun, int rowmodel, int colmodel, float colprune, 
              int nparts, int *part, int *ptree, int nfeatures, int *internalids,
              float *internalwgts, int *externalids, float *externalwgts);


void __cdecl CLUTO_InternalizeMatrix(int nrows, int ncols, int *rowptr, int *rowind,
              float *rowval, int simfun, int rowmodel, int colmodel, float colprune, 
              int *r_nrows, int *r_ncols, int **r_rowptr, int **r_rowind, 
              float **r_rowval, int **r_rimap, int **r_cimap);

void __cdecl CLUTO_S_TreeReorder(int nrows, int *rwgts, float *smat, int memflag,
              int dbglvl, int *ptree, int **ftree);

void __cdecl CLUTO_V_TreeReorder(int nrows, int ncols, int *rowptr, int *rowind,
              float *rowval, int simfun, int rowmodel, int colmodel, float colprune, 
              int dbglvl, int *ptree, int **ftree);

void __cdecl CLUTO_V_ClusterTreeReorder(int nrows, int ncols, int *rowptr, int *rowind,
              float *rowval, int simfun, int rowmodel, int colmodel, float colprune, 
              int dbglvl, int nparts, int *part, int *ptree, int **ftree);

#ifdef __cplusplus
}
#endif




/*------------------------------------------------------------------------
* Constant definitions 
*-------------------------------------------------------------------------*/

/* Different choices for RowModel */
#define CLUTO_ROWMODEL_NONE           1       /* Use the rows as is */
#define CLUTO_ROWMODEL_MAXTF          2       /* TF = .5 + .5(TF/max(|TF|)) */
#define CLUTO_ROWMODEL_SQRT           3       /* TF = 1+sign(TF, sqrt(|TF|)) */
#define CLUTO_ROWMODEL_LOG            4       /* TF = 1+sign(TF, log2(|TF|)) */


/* Different choices for ColModel */
#define CLUTO_COLMODEL_NONE           1       /* Use the columns as is */
#define CLUTO_COLMODEL_IDF            2       /* Scale-the columns following a binary IDF model */


/* Different cluster criterion functions */
#define CLUTO_CLFUN_I1      1       /* The I1 from the paper */
#define CLUTO_CLFUN_I2      2       /* The I2 from the paper */
#define CLUTO_CLFUN_E1      3       /* The E1 from the paper */
#define CLUTO_CLFUN_G1      4       /* The G1 from the paper */
#define CLUTO_CLFUN_G1P     5       /* The G1' from the paper */
#define CLUTO_CLFUN_H1      6       /* The H1 from the paper */
#define CLUTO_CLFUN_H2      7       /* The H2 from the paper */
#define CLUTO_CLFUN_H3      8       /* The I1*I2 from the paper */
#define CLUTO_CLFUN_G3      9       /* The n_r*G1 scaling from the paper */


/* Different dbglvl options */
#define CLUTO_DBG_PROGRESS         1       /* Show simple progress statistics */
#define CLUTO_DBG_RPROGRESS        2       /* Show simple progress statistics during refinement */
#define CLUTO_DBG_APROGRESS        4       /* Show progress during the agglomeration */


/* Different option for memory re-use for the SA-routines */
#define CLUTO_MEM_NOREUSE       1       /* Preserves the supplied information */
#define CLUTO_MEM_REUSE         2       /* Does not preserve the supplied information */

/* Different types of trees that CLUTO can build on top of the clustering solution */
#define CLUTO_TREE_TOP       1       /* Builds a tree on top of the supplied clustering */
#define CLUTO_TREE_FULL      2       /* Builds the entire tree that preserves the clustering */

/* Different similarity functions that CLUTO supports */
#define CLUTO_SIM_COSINE        1       /* Similarity is measured using the cosine function */
#define CLUTO_SIM_CORRCOEF      2       /* Similarity is measured using Pearson's correlatio coefficient */

/* Cluto's version number */
#define CLUTO_VER_MAJOR         1
#define CLUTO_VER_MINOR         5


#endif
